import logging
from pathlib import Path

import elasticsearch
from flask import Flask
from flask_compress import Compress
from flask_login import LoginManager

from archivy import helpers
from archivy.api import api_bp
from archivy.models import User
from archivy.config import Config
from archivy.helpers import load_config

app = Flask(__name__)
app.logger.setLevel(logging.INFO)
config = Config()
try:
    # if it exists, load user config
    config.override(load_config(config.INTERNAL_DIR))
except FileNotFoundError:
    pass

app.config.from_object(config)

(Path(app.config["USER_DIR"]) / "data").mkdir(parents=True, exist_ok=True)

if app.config["SEARCH_CONF"]["enabled"]:
    with app.app_context():
        es = helpers.get_elastic_client()
        try:
            es.indices.create(
                index=app.config["SEARCH_CONF"]["index_name"],
                body=app.config["SEARCH_CONF"]["search_conf"])
        except elasticsearch.exceptions.RequestError:
            app.logger.info("Elasticsearch index already created")


# login routes / setup
login_manager = LoginManager()
login_manager.login_view = "login"
login_manager.init_app(app)
app.register_blueprint(api_bp, url_prefix='/api')

# compress files
Compress(app)


@login_manager.user_loader
def load_user(user_id):
    db = helpers.get_db()
    res = db.get(doc_id=int(user_id))
    if res and res["type"] == "user":
        return User.from_db(res)
    return None


app.jinja_options["extensions"].append("jinja2.ext.do")

from archivy import routes  # noqa:



from html import escape
from typing import List, Tuple

import click
from flask import abort, render_template

from archivy import click_web
from archivy.click_web.exceptions import CommandNotFound
from archivy.click_web.resources.input_fields import get_input_field


def get_form_for(command_path: str):
    try:
        ctx_and_commands = _get_commands_by_path(command_path)
    except CommandNotFound as err:
        return abort(404, str(err))

    levels = _generate_form_data(ctx_and_commands)
    return render_template('click_web/command_form.html',
                           levels=levels,
                           command=levels[-1]['command'],
                           command_path=command_path)


def _get_commands_by_path(command_path: str) -> Tuple[click.Context, click.Command]:
    """
    Take a (slash separated) string and generate (context, command) for each level.
    :param command_path: "some_group/a_command"
    :return: Return a list from root to leaf commands.
    """
    command_path = "cli/" + command_path
    command_path_items = command_path.split('/')
    command_name, *command_path_items = command_path_items
    command = click_web.click_root_cmd
    if command.name != command_name:
        raise CommandNotFound(
                'Failed to find root command {}. There is a root commande named: {}'
                .format(command_name, command.name))
    result = []
    with click.Context(command, info_name=command, parent=None) as ctx:
        result.append((ctx, command))
        # dig down the path parts to find the leaf command
        parent_command = command
        for command_name in command_path_items:
            command = parent_command.get_command(ctx, command_name)
            if command:
                # create sub context for command
                ctx = click.Context(command, info_name=command, parent=ctx)
                parent_command = command
            else:
                raise CommandNotFound("""Failed to find command for path "{}".
                                       Command "{}" not found. Must be one of {}"""
                                      .format(command_path,
                                              command_name,
                                              parent_command.list_commands(ctx)))
            result.append((ctx, command))
    return result


def _generate_form_data(ctx_and_commands: List[Tuple[click.Context, click.Command]]):
    """
    Construct a list of contexts and commands generate a
    python data structure for rendering jinja form
    :return: a list of dicts
    """
    levels = []
    for command_index, (ctx, command) in enumerate(ctx_and_commands):
        # force help option off, no need in web.
        command.add_help_option = False
        command.html_help = _process_help(command.help)

        input_fields = [get_input_field(ctx, param, command_index, param_index)
                        for param_index, param in enumerate(command.get_params(ctx))]
        levels.append({'command': command, 'fields': input_fields})

    return levels


def _process_help(help_text):
    """
    Convert click command help into html to be presented to browser.
    Respects the '\b' char used by click to mark pre-formatted blocks.
    Also escapes html reserved characters in the help text.

    :param help_text: str
    :return: A html formatted help string.
    """
    help = []
    in_pre = False
    html_help = ''
    if not help_text:
        return html_help

    line_iter = iter(help_text.splitlines())
    while True:
        try:
            line = next(line_iter)
            if in_pre and not line.strip():
                # end of code block
                in_pre = False
                html_help += '\n'.join(help)
                help = []
                help.append('</pre>')
                continue
            elif line.strip() == '\b':
                # start of code block
                in_pre = True
                html_help += '<br>\n'.join(help)
                help = []
                help.append('<pre>')
                continue
            help.append(escape(line))
        except StopIteration:
            break

    html_help += '\n'.join(help) if in_pre else '<br>\n'.join(help)
    return html_help



from collections import OrderedDict

import click
from flask import render_template

from archivy import click_web


def index():
    with click.Context(click_web.click_root_cmd,
                       info_name=click_web.click_root_cmd.name, parent=None) as ctx:
        return render_template('click_web/show_tree.html',
                               ctx=ctx, tree=_click_to_tree(ctx, click_web.click_root_cmd),
                               title="Plugins")


def _click_to_tree(ctx: click.Context, node: click.BaseCommand, ancestors=[]):
    '''
    Convert a click root command to a tree of dicts and lists
    :return: a json like tree
    '''
    res_childs = []
    res = OrderedDict()
    res['is_group'] = isinstance(node, click.core.MultiCommand)
    omitted = ["shell", "run", "routes", "create-admin"]
    if res['is_group']:
        # a group, recurse for e    very child
        subcommand_names = set(click.Group.list_commands(node, ctx))
        children = [node.get_command(ctx, key) for key in subcommand_names
                    if key not in omitted]
        # Sort so commands comes before groups
        children = sorted(children, key=lambda c: isinstance(c, click.core.MultiCommand))
        for child in children:
            res_childs.append(_click_to_tree(ctx, child, ancestors[:] + [node, ]))

    res['name'] = node.name

    # Do not include any preformatted block (\b) for the short help.
    res['short_help'] = node.get_short_help_str().split('\b')[0]
    res['help'] = node.help
    path_parts = ancestors + [node]
    res['path'] = '/' + '/'.join(p.name for p in path_parts)
    if res_childs:
        res['childs'] = res_childs
    return res



import os
import appdirs


class Config(object):
    """Configuration object for the application"""

    def __init__(self):
        self.SECRET_KEY = os.urandom(32)
        self.PORT = 5000
        self.HOST = "127.0.0.1"
        self.INTERNAL_DIR = appdirs.user_data_dir("archivy")
        self.USER_DIR = self.INTERNAL_DIR
        os.makedirs(self.INTERNAL_DIR, exist_ok=True)

        self.PANDOC_HIGHLIGHT_THEME = "pygments"

        self.SEARCH_CONF = {
                "enabled": 0,
                "url": "http://localhost:9200",
                "index_name": "dataobj",
                "search_conf": {
                    "settings": {
                        "highlight": {
                            "max_analyzed_offset": 100000000
                        },
                        "analysis": {
                            "analyzer": {
                                "rebuilt_standard": {
                                    "stopwords": "_english_",
                                    "tokenizer": "standard",
                                    "filter": [
                                        "lowercase",
                                        "kstem",
                                        "trim",
                                        "unique"
                                    ]
                                }
                            }
                        }
                    },
                    "mappings": {
                        "properties": {
                            "title": {
                                "type": "text",
                                "analyzer": "rebuilt_standard",
                                "term_vector": "with_positions_offsets"
                            },
                            "tags": {"type": "text", "analyzer": "rebuilt_standard"},
                            "body": {"type": "text", "analyzer": "rebuilt_standard"},
                            "desc": {"type": "text", "analyzer": "rebuilt_standard"}
                        }
                    }
                }
            }

    def override(self, user_conf: dict):
        for k, v in user_conf.items():
            # handle ES options, don't override entire dict if one key is passed
            if k == "SEARCH_CONF":
                for subkey, subval in v.items():
                    self.SEARCH_CONF[subkey] = subval
            else:
                setattr(self, k, v)



from flask import current_app

from archivy.helpers import get_elastic_client


def add_to_index(model):
    """
    Adds dataobj to given index. If object of given id already exists, it will be updated.

    Params:

    - **index** - String of the ES Index. Archivy uses `dataobj` by default.
    - **model** - Instance of `archivy.models.Dataobj`, the object you want to index.
    """
    es = get_elastic_client()
    if not es:
        return
    payload = {}
    for field in model.__searchable__:
        payload[field] = getattr(model, field)
    es.index(index=current_app.config["SEARCH_CONF"]["index_name"], id=model.id, body=payload)
    return True


def remove_from_index(dataobj_id):
    """Removes object of given id"""
    es = get_elastic_client()
    if not es:
        return
    es.delete(index=current_app.config["SEARCH_CONF"]["index_name"], id=dataobj_id)


def query_index(query):
    """Returns search results for your given query"""
    es = get_elastic_client()
    if not es:
        return []
    search = es.search(
        index=current_app.config["SEARCH_CONF"]["index_name"],
        body={
            "query": {
                "multi_match": {
                    "query": query,
                    "fields": ["*"],
                    "analyzer": "rebuilt_standard"
                }
            },
            "highlight": {
                "fragment_size": 0,
                "fields": {
                    "content": {
                        "pre_tags": "==",
                        "post_tags": "==",
                    }
                }
            }
        }
    )

    hits = []
    for hit in search["hits"]["hits"]:
        formatted_hit = {"id": hit["_id"], "title": hit["_source"]["title"]}
        if "highlight" in hit:
            formatted_hit["highlight"] = hit["highlight"]["content"]
        hits.append(formatted_hit)

    return hits



import tempfile
from pathlib import Path

import click
import jinja2
from flask import Blueprint

from archivy.click_web.resources import cmd_exec, cmd_form, index

jinja_env = jinja2.Environment(extensions=['jinja2.ext.do'])

'The full path to the click script file to execute.'
script_file = None
'The click root command to serve'
click_root_cmd = None


def _get_output_folder():
    _output_folder = (Path(tempfile.gettempdir()) / 'click-web')
    if not _output_folder.exists():
        _output_folder.mkdir()
    return _output_folder


'Where to place result files for download'
OUTPUT_FOLDER = str(_get_output_folder())

_flask_app = None
logger = None


def create_click_web_app(module, command: click.BaseCommand, flask_app):
    '''
    Create a Flask app that wraps a click command. (Call once)

    :param module: the module that contains the click command,
    needed to get the path to the script.
    :param command: The actual click root command, needed
    to be able to read the command tree and arguments
    in order to generate the index page and the html forms
    '''
    global _flask_app, logger
    assert _flask_app is None, "Flask App already created."

    _register(module, command)

    _flask_app = flask_app

    # add the "do" extension needed by our jinja templates
    _flask_app.jinja_env.add_extension('jinja2.ext.do')

    _flask_app.add_url_rule('/plugins', 'cli_index', index.index)
    _flask_app.add_url_rule('/cli/<path:command_path>', 'command', cmd_form.get_form_for)
    _flask_app.add_url_rule('/cli/<path:command_path>', 'command_execute', cmd_exec.exec,
                            methods=['POST'])

    _flask_app.logger.info(f'OUTPUT_FOLDER: {OUTPUT_FOLDER}')
    results_blueprint = Blueprint('results',
                                  __name__,
                                  static_url_path='/static/results',
                                  static_folder=OUTPUT_FOLDER)
    _flask_app.register_blueprint(results_blueprint)

    logger = _flask_app.logger

    return _flask_app


def _register(module, command: click.BaseCommand):
    '''
    :param module: the module that contains the command, needed to get the path to the script.
    :param command: The actual click root command,
                    needed to be able to read the command tree and arguments
                    in order to generate the index page and the html forms
    '''
    global click_root_cmd, script_file
    script_file = str(Path(module.__file__).absolute())
    click_root_cmd = command



import click

from archivy.click_web.web_click_types import EmailParamType, PasswordParamType


class FieldId:
    """
    Extract/serialize information from the encoded form input field name
    the parts:
        [command_index].[opt_or_arg_index].[click_type].[html_input_type].[opt_or_arg_name]
    e.g.
        "0.0.option.text.text.--an-option"
        "0.1.argument.file[rb].text.an-argument"
    """
    SEPARATOR = '.'

    def __init__(self,
                 command_index,
                 param_index,
                 param_type,
                 click_type,
                 nargs,
                 form_type,
                 name,
                 key=None):
        'the int index of the command it belongs to'
        self.command_index = int(command_index)
        'the int index for the ordering of paramters/arguments'
        self.param_index = int(param_index)
        'Type of option (argument, option, flag)'
        self.param_type = param_type
        'Type of option (file, text)'
        self.click_type = click_type
        'nargs value (-1 is variardic)'
        self.nargs = int(nargs)
        'Type of html input type'
        self.form_type = form_type
        'The actual command line option (--debug)'
        self.name = name
        'The actual form id'
        self.key = key if key else str(self)

    def __str__(self):
        return self.SEPARATOR.join(str(p) for p in (self.command_index,
                                                    self.param_index,
                                                    self.param_type,
                                                    self.click_type,
                                                    self.nargs,
                                                    self.form_type,
                                                    self.name))

    @classmethod
    def from_string(cls, field_info_as_string) -> 'FieldId':
        args = field_info_as_string.split(cls.SEPARATOR) + [field_info_as_string, ]
        return cls(*args)


class NotSupported(ValueError):
    pass


class BaseInput:
    param_type_cls = None

    def __init__(self, ctx, param: click.Parameter, command_index, param_index):
        self.ctx = ctx
        self.param = param
        self.command_index = command_index
        self.param_index = param_index
        if not self.is_supported():
            raise NotSupported()

    def is_supported(self):
        return isinstance(self.param.type, self.param_type_cls)

    @property
    def fields(self) -> dict:
        field = {}
        param = self.param

        field['param'] = param.param_type_name
        if param.param_type_name == 'option':
            name = '--{}'.format(self._to_cmd_line_name(param.name))
            field['value'] = param.default if param.default else ''
            field['checked'] = 'checked="checked"' if param.default else ''
            field['desc'] = param.help
            field['help'] = param.get_help_record(self.ctx)
        elif param.param_type_name == 'argument':
            name = self._to_cmd_line_name(param.name)
            field['value'] = param.default
            field['checked'] = ''
            field['help'] = ''

        field['name'] = self._build_name(name)
        field['required'] = param.required
        field['nargs'] = param.nargs
        field['human_readable_name'] = param.human_readable_name.replace('_', ' ')
        field.update(self.type_attrs)
        return field

    @property
    def type_attrs(self) -> dict:
        """
        Here the input type and type specific information should be retuned as a dict
        :return:
        """
        raise NotImplementedError()

    def _to_cmd_line_name(self, name: str) -> str:
        return name.replace('_', '-')

    def _build_name(self, name: str):
        """
        Construct a name to use for field in form that have information about
        what sub-command it belongs order index (for later sorting) and type of parameter.
        """
        # get the type of param to encode the in the name
        if self.param.param_type_name == 'option':
            param_type = 'flag' if self.param.is_bool_flag else 'option'
        else:
            param_type = self.param.param_type_name

        click_type = self.type_attrs['click_type']
        form_type = self.type_attrs['type']

        # in order for form to be have arguments for sub commands we need to add the
        # index of the command the argument belongs to
        return str(FieldId(self.command_index,
                           self.param_index,
                           param_type,
                           click_type,
                           self.param.nargs,
                           form_type,
                           name))


class ChoiceInput(BaseInput):
    param_type_cls = click.Choice

    @property
    def type_attrs(self):
        type_attrs = {}
        type_attrs['type'] = 'option'
        type_attrs['options'] = self.param.type.choices
        type_attrs['default'] = self.param.default
        type_attrs['click_type'] = 'choice'
        return type_attrs


class FlagInput(BaseInput):
    def is_supported(self, ):
        return self.param.param_type_name == 'option' and self.param.is_bool_flag

    @property
    def type_attrs(self):
        type_attrs = {}
        type_attrs['type'] = 'checkbox'
        type_attrs['click_type'] = 'bool_flag'
        type_attrs['value'] = self.param.opts[0]
        # the "on" value e.g ['--flag']
        type_attrs['on_flag'] = self.param.opts[0]
        # the "off" value e.g ['--no-flag']
        if self.param.secondary_opts:
            type_attrs['off_flag'] = self.param.secondary_opts[0]
        return type_attrs


class IntInput(BaseInput):
    param_type_cls = click.types.IntParamType

    @property
    def type_attrs(self):
        type_attrs = {}
        type_attrs['type'] = 'number'
        type_attrs['step'] = '1'
        type_attrs['click_type'] = 'int'
        return type_attrs


class FloatInput(BaseInput):
    param_type_cls = click.types.FloatParamType

    @property
    def type_attrs(self):
        type_attrs = {}
        type_attrs['type'] = 'number'
        type_attrs['step'] = 'any'
        type_attrs['click_type'] = 'float'
        return type_attrs


class FolderInput(BaseInput):

    def is_supported(self):
        if isinstance(self.param.type, click.Path):
            if self.param.type.dir_okay:
                return True
        return False

    @property
    def type_attrs(self):
        type_attrs = {}
        # if it is required we treat it as an input folder
        # and only accept zip.
        mode = 'r' if self.param.type.exists else 'w'
        type_attrs['click_type'] = f'path[{mode}]'
        if self.param.type.exists:
            type_attrs['accept'] = "application/zip"
            type_attrs['type'] = 'file'
        else:
            type_attrs['type'] = 'hidden'
        return type_attrs


class FileInput(BaseInput):

    def is_supported(self):
        if isinstance(self.param.type, click.File):
            return True
        elif isinstance(self.param.type, click.Path):
            if (self.param.type.file_okay):
                return True
        return False

    @property
    def type_attrs(self):
        type_attrs = {}
        if isinstance(self.param.type, click.File):
            mode = self.param.type.mode
        elif isinstance(self.param.type, click.Path):
            mode = 'w' if self.param.type.writable else ''
            mode += 'r' if self.param.type.readable else ''
        else:
            raise NotSupported(f'Illegal param type. Got type: {self.param.type}')

        type_attrs['click_type'] = f'file[{mode}]'

        if 'r' not in mode:
            if self.param.required:
                # if file is only for output do not show in form
                type_attrs['type'] = 'hidden'
            else:
                type_attrs['type'] = 'text'

        else:
            type_attrs['type'] = 'file'
        return type_attrs


class EmailInput(BaseInput):
    param_type_cls = EmailParamType

    @property
    def type_attrs(self):
        type_attrs = {}
        type_attrs['type'] = 'email'
        type_attrs['click_type'] = 'email'
        return type_attrs


class PasswordInput(BaseInput):
    param_type_cls = PasswordParamType

    @property
    def type_attrs(self):
        type_attrs = {}
        type_attrs['type'] = 'password'
        type_attrs['click_type'] = 'password'
        return type_attrs


class DefaultInput(BaseInput):
    param_type_cls = click.ParamType

    @property
    def type_attrs(self):
        type_attrs = {}
        type_attrs['type'] = 'text'
        type_attrs['click_type'] = 'text'
        return type_attrs


'''
The types of inputs we support form inputs listed in priority order
(first that matches will be selected).
To add new Input handling for html forms for custom Parameter types
just Subclass BaseInput and insert the class in the list.
'''
INPUT_TYPES = [ChoiceInput,
               FlagInput,
               IntInput,
               FloatInput,
               FolderInput,
               FileInput,
               EmailInput,
               PasswordInput]

_DEFAULT_INPUT = [DefaultInput]


def get_input_field(ctx: click.Context,
                    param: click.Parameter,
                    command_index, param_index) -> dict:
    """
    Convert a click.Parameter into a dict structure describing a html form option
    """
    for input_cls in INPUT_TYPES + _DEFAULT_INPUT:
        try:
            input_type = input_cls(ctx, param, command_index, param_index)
        except NotSupported:
            pass
        else:
            fields = input_type.fields
            return fields
    raise NotSupported(f"No Form input type not supported: {param}")



import platform
import subprocess
import os
from pathlib import Path
from shutil import rmtree

import frontmatter
from flask import current_app
from werkzeug.utils import secure_filename

from archivy.search import remove_from_index


# FIXME: ugly hack to make sure the app path is evaluated at the right time
def get_data_dir():
    """Returns the directory where dataobjs are stored"""
    return Path(current_app.config['USER_DIR']) / "data"


class Directory:
    """Tree like file-structure used to build file navigation in Archiv"""
    def __init__(self, name):
        self.name = name
        self.child_files = []
        self.child_dirs = {}


FILE_GLOB = "-[0-9][0-9]-[0-9][0-9]-[0-9][0-9]-*"


def get_by_id(dataobj_id):
    """Returns filename of dataobj of given id"""
    results = list(get_data_dir().rglob(f"{dataobj_id}{FILE_GLOB}"))
    return results[0] if results else None


def get_items(collections=[], path="", structured=True, json_format=False):
    """
    Gets all dataobjs.

    Parameters:

    - **collections** - filter dataobj by type, eg. bookmark / note
    - **path** - filter by path
    - **structured: if set to True, will return a Directory object, otherwise
      data will just be returned as a list of dataobjs
    - **json_format**: boolean value used internally to pre-process dataobjs
      to send back a json response.
    """
    datacont = Directory("root") if structured else []
    home_dir = get_data_dir()
    for filename in home_dir.rglob(path + "*"):
        if structured:
            paths = filename.relative_to(home_dir)
            current_dir = datacont

            # iterate through paths
            for segment in paths.parts:
                if segment.endswith(".md"):
                    data = frontmatter.load(filename)
                    current_dir.child_files.append(data)
                else:
                    # directory has not been saved in tree yet
                    if segment not in current_dir.child_dirs:
                        current_dir.child_dirs[segment] = Directory(segment)
                    current_dir = current_dir.child_dirs[segment]
        else:
            if filename.parts[-1].endswith(".md"):
                data = frontmatter.load(filename)
                if len(collections) == 0 or \
                        any([collection == data["type"]
                            for collection in collections]):
                    if json_format:
                        dict_dataobj = data.__dict__
                        # remove unnecessary yaml handler
                        dict_dataobj.pop("handler")
                        datacont.append(dict_dataobj)
                    else:
                        datacont.append(data)
    return datacont


def create(contents, title, path=""):
    """
    Helper method to save a new dataobj onto the filesystem.

    Parameters:

    - **contents**: md file contents
    - **title** - title used for filename
    - **path**
    """
    path_to_md_file = get_data_dir() / path / f"{secure_filename(title)}.md"
    with open(path_to_md_file, "w", encoding="utf-8") as file:
        file.write(contents)

    return path_to_md_file


def get_item(dataobj_id):
    """Returns a Post object with the given dataobjs' attributes"""
    file = get_by_id(dataobj_id)
    if file:
        data = frontmatter.load(file)
        data["fullpath"] = str(file)
        return data
    return None


def delete_item(dataobj_id):
    """Delete dataobj of given id"""
    file = get_by_id(dataobj_id)
    remove_from_index(dataobj_id)
    if file:
        Path(file).unlink()


def update_item(dataobj_id, new_content):
    """
    Given an object id, this method overwrites the inner
    content of the post with `new_content`.

    This means that it won't change the frontmatter (eg tags, id, title)
    but it can change the file content.

    For example:

    If we have a dataobj like this:

    ```md
    ---
    id: 1
    title: Note
    ---

    # This is random
    ```

    Calling `update_item(1, "# This is specific")` will turn it into:


    ```md
    ---
    id: 1 # unchanged
    title: Note
    ---

    # This is specific
    ```
    """
    from archivy.models import DataObj
    filename = get_by_id(dataobj_id)
    dataobj = frontmatter.load(filename)
    dataobj.content = new_content
    md = frontmatter.dumps(dataobj)
    with open(filename, "w", encoding="utf-8") as f:
        f.write(md)

    # save changes to ES
    DataObj.from_md(md).index()


def get_dirs():
    """Gets all dir names where dataobjs are stored"""
    # join glob matchers
    dirnames = [str(dir_path.relative_to(get_data_dir())) for dir_path
                in get_data_dir().rglob("*") if dir_path.is_dir()]

    # append name for root dir
    dirnames.append("not classified")
    return dirnames


def create_dir(name):
    """Create dir of given name"""
    home_dir = get_data_dir()
    new_path = home_dir / name
    new_path.mkdir(parents=True, exist_ok=True)
    return str(new_path.relative_to(home_dir))


def delete_dir(name):
    """Deletes dir of given name"""
    try:
        rmtree(get_data_dir() / name)
        return True
    except FileNotFoundError:
        return False


def format_file(path: str):
    """
    Converts normal md of file at `path` to formatted archivy markdown file, with yaml front matter
    and a filename of format "{id}-{old_filename}.md"
    """

    from archivy.models import DataObj
    data_dir = get_data_dir()
    path = Path(path)
    if not path.exists():
        return

    if path.is_dir():
        for filename in path.iterdir():
            format_file(filename)

    else:
        new_file = path.open()
        file_contents = new_file.read()
        new_file.close()
        try:
            # get relative path of object in `data` dir
            datapath = path.parent.resolve().relative_to(data_dir)
        except ValueError:
            datapath = Path()

        note_dataobj = {
                "title": path.name.replace(".md", ""),
                "content": file_contents,
                "type": "note",
                "path": str(datapath)
            }

        dataobj = DataObj(**note_dataobj)
        dataobj.insert()

        path.unlink()
        current_app.logger.info(
                f"Formatted and moved {str(datapath / path.name)} to {dataobj.fullpath}")


def unformat_file(path: str, out_dir: str):
    """
    Converts normal md of file at `path` to formatted archivy markdown file, with yaml front matter
    and a filename of format "{id}-{old_filename}.md"
    """

    data_dir = get_data_dir()
    path = Path(path)
    out_dir = Path(out_dir)
    if not path.exists() and out_dir.exists() and out_dir.is_dir():
        return

    if path.is_dir():
        path.mkdir(exist_ok=True)
        for filename in path.iterdir():
            unformat_file(filename, str(out_dir))

    else:
        dataobj = frontmatter.load(str(path))

        try:
            # get relative path of object in `data` dir
            datapath = path.parent.resolve().relative_to(data_dir)
        except ValueError:
            datapath = Path()

        # create subdir if doesn't exist
        (out_dir / datapath).mkdir(exist_ok=True)
        new_path = out_dir / datapath / f"{dataobj.metadata['title']}.md"
        with new_path.open("w") as f:
            f.write(dataobj.content)

        current_app.logger.info(f"Unformatted and moved {str(path)} to {str(new_path.resolve())}")
        path.unlink()


def open_file(path):
    """Cross platform way of opening file on user's computer"""
    if platform.system() == "Windows":
        os.startfile(path)
    elif platform.system() == "Darwin":
        subprocess.Popen(["open", path])
    else:
        subprocess.Popen(["xdg-open", path])



from flask import Response, jsonify, request, Blueprint, current_app
from werkzeug.security import check_password_hash
from flask_login import login_user
from tinydb import Query

from archivy import data
from archivy.search import query_index
from archivy.models import DataObj, User
from archivy.helpers import get_db


api_bp = Blueprint('api', __name__)


@api_bp.route("/login", methods=["POST"])
def login():
    """
    Logs in the API client using
    [HTTP Basic Auth](https://en.wikipedia.org/wiki/Basic_access_authentication).
    Pass in the username and password of your account.
    """
    db = get_db()
    user = db.search(Query().username == request.authorization["username"])
    if (user and
            check_password_hash(
                user[0]["hashed_password"],
                request.authorization["password"])):
        # user is verified so we can log him in from the db
        user = User.from_db(user[0])
        login_user(user, remember=True)
        return Response(status=200)
    return Response(status=401)


@api_bp.route("/bookmarks", methods=["POST"])
def create_bookmark():
    """
    Creates a new bookmark

    **Parameters:**

    All parameters are sent through the JSON body.
    - **url** (required)
    - **desc**
    - **tags**
    - **path**
    """
    json_data = request.get_json()
    bookmark = DataObj(
        url=json_data['url'],
        desc=json_data.get('desc'),
        tags=json_data.get('tags'),
        path=json_data.get("path", ""),
        type="bookmark",
    )
    bookmark.process_bookmark_url()
    bookmark_id = bookmark.insert()
    if bookmark_id:
        return jsonify(
            bookmark_id=bookmark_id,
        )
    return Response(status=400)


@api_bp.route("/notes", methods=["POST"])
def create_note():
    """
    Creates a new note.

    **Parameters:**

    All parameters are sent through the JSON body.
    - **title** (required)
    - **content** (required)
    - **desc**
    - **tags**
    - **path**
    """
    json_data = request.get_json()
    note = DataObj(
        title=json_data["title"],
        content=json_data["content"],
        desc=json_data.get("desc"),
        tags=json_data.get("tags"),
        path=json_data.get("path", ""),
        type="note"
    )

    note_id = note.insert()
    if note_id:
        return jsonify(note_id=note_id)
    return Response(status=400)


@api_bp.route("/dataobjs/<int:dataobj_id>")
def get_dataobj(dataobj_id):
    """Returns dataobj of given id"""
    dataobj = data.get_item(dataobj_id)

    return jsonify(
        dataobj_id=dataobj_id,
        title=dataobj["title"],
        content=dataobj.content,
        md_path=dataobj["fullpath"],
    ) if dataobj else Response(status=404)


@api_bp.route("/bookmarks/<int:bookmark_id>", methods=["PUT"])
def change_bookmark(bookmark_id):
    current_app.logger.debug(f'Attempting to delete bookmark <{bookmark_id}>')
    return Response(status=501)


@api_bp.route("/dataobjs/<int:dataobj_id>", methods=["DELETE"])
def delete_dataobj(dataobj_id):
    """Deletes object of given id"""
    if not data.get_item(dataobj_id):
        return Response(status=404)
    data.delete_item(dataobj_id)
    return Response(status=204)


@api_bp.route("/dataobjs/<int:dataobj_id>", methods=["PUT"])
def update_dataobj(dataobj_id):
    """
    Updates object of given id.

    Paramter in JSON body:

    - **content**: markdown text of new dataobj.
    """
    if request.json.get("content"):
        try:
            data.update_item(dataobj_id, request.json.get("content"))
            return Response(status=200)
        except BaseException:
            return Response(status=404)
    return Response("Must provide content parameter", status=401)


@api_bp.route("/dataobjs", methods=["GET"])
def get_dataobjs():
    """Gets all dataobjs"""
    cur_dir = data.get_items(structured=False, json_format=True)
    return jsonify(cur_dir)


@api_bp.route("/dataobj/local_edit/<dataobj_id>", methods=["GET"])
def local_edit(dataobj_id):
    dataobj = data.get_item(dataobj_id)
    if dataobj:
        data.open_file(dataobj["fullpath"])
        return Response(status=200)
    return Response(status=404)


@api_bp.route("/folders/new", methods=["POST"])
def create_folder():
    """
    Creates new directory

    Parameter in JSON body:
    - **path** (required) - path of newdir
    """
    directory = request.json.get("path")
    try:
        sanitized_name = data.create_dir(directory)
    except FileExistsError:
        return Response("Directory already exists", status=401)
    return Response(sanitized_name, status=200)


@api_bp.route("/folders/delete", methods=["DELETE"])
def delete_folder():
    """
    Deletes directory.

    Parameter in JSON body:
    - **path** of dir to delete
    """
    directory = request.json.get("path")
    if directory == "":
        return Response("Cannot delete root dir", status=401)
    if data.delete_dir(directory):
        return Response("Successfully deleted", status=200)
    return Response("Not found", status=404)


@api_bp.route("/search", methods=["GET"])
def search_elastic():
    """
    Searches the instance.

    Request URL Parameter:
    - **query**
    """
    query = request.args.get("query")
    search_results = query_index(query)
    return jsonify(search_results)



import pprint
from pathlib import Path

import click
import pytest

from archivy import click_web, cli
from archivy.click_web.resources import cmd_form, input_fields


@pytest.mark.parametrize(
    'param, command_index, expected',
    [
        (click.Argument(["an_argument", ]), 0,
         {'checked': '',
          'click_type': 'text',
          'help': '',
          'human_readable_name': 'AN ARGUMENT',
          'name': '0.0.argument.text.1.text.an-argument',
          'nargs': 1,
          'param': 'argument',
          'required': True,
          'type': 'text',
          'value': None}),
        (click.Argument(["an_argument", ], nargs=2), 1,
         {'checked': '',
          'click_type': 'text',
          'help': '',
          'human_readable_name': 'AN ARGUMENT',
          'name': '1.0.argument.text.2.text.an-argument',
          'nargs': 2,
          'param': 'argument',
          'required': True,
          'type': 'text',
          'value': None}),
        (click.Option(["--an_option", ]), 0,
         {'checked': '',
          'click_type': 'text',
          'desc': None,
          'help': ('--an_option TEXT', ''),
          'human_readable_name': 'an option',
          'name': '0.0.option.text.1.text.--an-option',
          'nargs': 1,
          'param': 'option',
          'required': False,
          'type': 'text',
          'value': ''}),
        (click.Option(["--an_option", ], nargs=2), 1,
         {'checked': '',
          'click_type': 'text',
          'desc': None,
          'help': ('--an_option TEXT...', ''),
          'human_readable_name': 'an option',
          'name': '1.0.option.text.2.text.--an-option',
          'nargs': 2,
          'param': 'option',
          'required': False,
          'type': 'text',
          'value': ''}),
        (click.Option(["--flag/--no-flag", ], default=True, help='help'), 3,
         {'checked': 'checked="checked"',
          'click_type': 'bool_flag',
          'desc': 'help',
          'help': ('--flag / --no-flag', 'help'),
          'human_readable_name': 'flag',
          'name': '3.0.flag.bool_flag.1.checkbox.--flag',
          'nargs': 1,
          'off_flag': '--no-flag',
          'on_flag': '--flag',
          'param': 'option',
          'required': False,
          'type': 'checkbox',
          'value': '--flag'}),
    ])
def test_get_input_field(ctx, click_cli, param, expected, command_index):
    res = input_fields.get_input_field(ctx, param, command_index, 0)
    pprint.pprint(res)
    assert res == expected


@pytest.mark.parametrize(
    'param, command_index, expected',
    [
        (click.Argument(["an_argument", ], nargs=-1), 0,
         {'checked': '',
          'click_type': 'text',
          'help': '',
          'human_readable_name': 'AN ARGUMENT',
          'name': '0.0.argument.text.-1.text.an-argument',
          'nargs': -1,
          'param': 'argument',
          'required': False,
          'type': 'text',
          'value': None}),
    ])
def test_variadic_arguments(ctx, click_cli, param, expected, command_index):
    res = input_fields.get_input_field(ctx, param, command_index, 0)
    pprint.pprint(res)
    assert res == expected


@pytest.mark.parametrize(
    'param, command_index, expected',
    [
        (click.Argument(["a_file_argument", ], type=click.File('rb')), 0,
         {'checked': '',
          'click_type': 'file[rb]',
          'help': '',
          'human_readable_name': 'A FILE ARGUMENT',
          'name': '0.0.argument.file[rb].1.file.a-file-argument',
          'nargs': 1,
          'param': 'argument',
          'required': True,
          'type': 'file',
          'value': None}),
        (click.Option(["--a_file_option", ], type=click.File('rb')), 0,
         {'checked': '',
          'click_type': 'file[rb]',
          'desc': None,
          'help': ('--a_file_option FILENAME', ''),
          'human_readable_name': 'a file option',
          'name': '0.0.option.file[rb].1.file.--a-file-option',
          'nargs': 1,
          'param': 'option',
          'required': False,
          'type': 'file',
          'value': ''}),
    ])
def test_get_file_input_field(ctx, click_cli, param, expected, command_index):
    res = input_fields.get_input_field(ctx, param, command_index, 0)
    pprint.pprint(res)
    assert res == expected


@pytest.mark.parametrize(
    'param, command_index, expected',
    [
        (click.Argument(["a_file_argument", ], type=click.File('wb')), 0,
         {'checked': '',
          'click_type': 'file[wb]',
          'help': '',
          'human_readable_name': 'A FILE ARGUMENT',
          'name': '0.0.argument.file[wb].1.hidden.a-file-argument',
          'nargs': 1,
          'param': 'argument',
          'required': True,
          'type': 'hidden',
          'value': None}),
        (click.Option(["--a_file_option", ], type=click.File('wb')), 0,
         {'checked': '',
          'click_type': 'file[wb]',
          'desc': None,
          'help': ('--a_file_option FILENAME', ''),
          'human_readable_name': 'a file option',
          'name': '0.0.option.file[wb].1.text.--a-file-option',
          'nargs': 1,
          'param': 'option',
          'required': False,
          'type': 'text',
          'value': ''}),
    ])
def test_get_output_file_input_field(ctx, click_cli, param, expected, command_index):
    res = input_fields.get_input_field(ctx, param, command_index, 0)
    pprint.pprint(res)
    assert res == expected



from flask_wtf import FlaskForm
from wtforms import StringField, SubmitField, SelectField, PasswordField
from wtforms.validators import DataRequired, URL


class NewBookmarkForm(FlaskForm):
    url = StringField("url", validators=[DataRequired(), URL()])
    path = SelectField("Topic")
    desc = StringField("desc")
    tags = StringField("tags")
    submit = SubmitField("Save")


class NewNoteForm(FlaskForm):
    title = StringField("title", validators=[DataRequired()])
    path = SelectField("Topic")
    desc = StringField("desc")
    tags = StringField("tags")
    submit = SubmitField("Save")


class DeleteDataForm(FlaskForm):
    submit = SubmitField("Delete")


class UserForm(FlaskForm):
    username = StringField("username")
    password = PasswordField("password")
    submit = SubmitField("Submit")



"""
Extra click types that could be useful in a web context as
they have corresponding HTML form input type.
The custom web click types need only be imported
into the main script, not the app.py that flask runs.

Example usage in your click command:
\b
    from click_web.web_click_types import EMAIL_TYPE
    @cli.command()
    @click.option("--the_email", type=EMAIL_TYPE)
    def email(the_email):
        click.echo(f"{the_email} is a valid email syntax.")

"""
import re

import click


class EmailParamType(click.ParamType):
    name = 'email'
    EMAIL_REGEX = re.compile(r"(^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$)")

    def convert(self, value, param, ctx):
        if self.EMAIL_REGEX.match(value):
            return value
        else:
            self.fail(f'{value} is not a valid email', param, ctx)


class PasswordParamType(click.ParamType):
    name = "password"

    def convert(self, value, param, ctx):
        return value


EMAIL_TYPE = EmailParamType()
PASSWORD_TYPE = PasswordParamType()



from pathlib import Path
from os import environ
from pkg_resources import iter_entry_points

import click
from click_plugins import with_plugins
from flask.cli import FlaskGroup, load_dotenv, shell_command

from archivy import app
from archivy.config import Config
from archivy.click_web import create_click_web_app
from archivy.data import open_file, format_file, unformat_file
from archivy.helpers import load_config, write_config
from archivy.models import User, DataObj


def create_app():
    return app


@with_plugins(iter_entry_points('archivy.plugins'))
@click.group(cls=FlaskGroup, create_app=create_app)
def cli():
    pass


# add built in commands:
cli.add_command(shell_command)


@cli.command("init", short_help="Initialise your archivy application")
@click.pass_context
def init(ctx):
    try:
        load_config()
        click.confirm("Config already found. Do you wish to reset it? "
                      "Otherwise run `archivy config`", abort=True)
    except FileNotFoundError:
        pass

    config = Config()
    delattr(config, "SECRET_KEY")

    click.echo("This is the archivy installation initialization wizard.")
    data_dir = click.prompt("Enter the full path of the "
                            "directory where you'd like us to store data.",
                            type=str,
                            default=str(Path(".").resolve()))

    es_enabled = click.confirm("Would you like to enable Elasticsearch? For this to work "
                               "when you run archivy, you must have ES installed."
                               "See https://archivy.github.io/setup-search/ for more info.")
    if es_enabled:
        config.SEARCH_CONF["enabled"] = 1
    else:
        delattr(config, "SEARCH_CONF")

    create_new_user = click.confirm("Would you like to create a new admin user?")
    if create_new_user:
        username = click.prompt("Username")
        password = click.prompt("Password", hide_input=True, confirmation_prompt=True)
        if not ctx.invoke(create_admin, username=username, password=password):
            return

    config.HOST = click.prompt("Host [localhost (127.0.0.1)]",
                               type=str, default="127.0.0.1", show_default=False)

    config.override({"USER_DIR": data_dir})
    app.config["USER_DIR"] = data_dir

    # create data dir
    (Path(data_dir) / "data").mkdir(exist_ok=True)

    write_config(vars(config))
    click.echo("Config successfully created at "
               + str((Path(app.config['INTERNAL_DIR']) / 'config.yml').resolve()))


@cli.command("config", short_help="Open archivy config.")
def config():
    open_file(str(Path(app.config["INTERNAL_DIR"]) / "config.yml"))


@cli.command("run", short_help="Runs archivy web application")
def run():
    click.echo('Running archivy...')
    load_dotenv()
    environ["FLASK_RUN_FROM_CLI"] = "false"
    app_with_cli = create_click_web_app(click, cli, app)
    app_with_cli.run(host=app.config["HOST"], port=app.config["PORT"])


@cli.command(short_help="Creates a new admin user")
@click.argument("username")
@click.password_option()
def create_admin(username, password):
    if len(password) < 8:
        click.echo("Password length too short")
        return False
    else:
        user = User(username=username, password=password, is_admin=True)
        if user.insert():
            click.echo(f"User {username} successfully created.")
            return True
        else:
            click.echo("User with given username already exists.")
            return False


@cli.command(short_help="Format normal markdown files for archivy.")
@click.argument("filenames", type=click.Path(exists=True), nargs=-1)
def format(filenames):
    for path in filenames:
        format_file(path)


@cli.command(short_help="Convert archivy-formatted files back to normal markdown.")
@click.argument("filenames", type=click.Path(exists=True), nargs=-1)
@click.argument("output_dir", type=click.Path(exists=True, file_okay=False))
def unformat(filenames, output_dir):
    for path in filenames:
        unformat_file(path, output_dir)


@cli.command(short_help="Sync content to Elasticsearch")
def index():
    data_dir = Path(app.config["USER_DIR"]) / "data"

    if not app.config["SEARCH_CONF"]["enabled"]:
        click.echo("Search must be enabled for this command.")
        return

    for filename in data_dir.rglob("*.md"):
        cur_file = open(filename)
        dataobj = DataObj.from_md(cur_file.read())
        cur_file.close()

        if dataobj.index():
            click.echo(f"Indexed {dataobj.title}...")
        else:
            click.echo(f"Failed to index {dataobj.title}")



import os

import frontmatter
from flask import render_template, flash, redirect, request, url_for
from flask_login import login_user, login_required, current_user, logout_user
from tinydb import Query
from werkzeug.security import check_password_hash, generate_password_hash

from archivy.models import DataObj, User
from archivy import data, app, forms
from archivy.helpers import get_db


@app.context_processor
def pass_defaults():
    dataobjs = data.get_items()
    SEP = os.path.sep
    # check windows parsing for js (https://github.com/Uzay-G/archivy/issues/115)
    if SEP == "\\":
        SEP += "\\"
    return dict(dataobjs=dataobjs, SEP=os.path.sep)


@app.before_request
def check_perms():
    allowed_path = (request.path.startswith("/login") or
                    request.path.startswith("/static") or
                    request.path.startswith("/api/login"))
    if not current_user.is_authenticated and not allowed_path:
        return redirect(url_for("login", next=request.path))
    return


@app.route("/")
@app.route("/index")
def index():
    return render_template(
            "home.html",
            title="Home",
            search_enabled=app.config["SEARCH_CONF"]["enabled"],
            )


# TODO: refactor two following methods
@app.route("/bookmarks/new", methods=["GET", "POST"])
def new_bookmark():
    form = forms.NewBookmarkForm()
    form.path.choices = [(pathname, pathname) for pathname in data.get_dirs()]
    if form.validate_on_submit():
        path = form.path.data if form.path.data != "not classified" else ""
        tags = form.tags.data.split(",") if form.tags.data != "" else []
        bookmark = DataObj(
            url=form.url.data,
            desc=form.desc.data,
            tags=tags,
            path=path,
            type="bookmark")
        bookmark.process_bookmark_url()
        bookmark_id = bookmark.insert()
        if bookmark_id:
            flash("Bookmark Saved!")
            return redirect(f"/dataobj/{bookmark_id}")
    return render_template(
        "dataobjs/new.html",
        title="New Bookmark",
        form=form)


@app.route("/notes/new", methods=["GET", "POST"])
def new_note():
    form = forms.NewNoteForm()
    form.path.choices = [(pathname, pathname) for pathname in data.get_dirs()]
    if form.validate_on_submit():
        path = form.path.data if form.path.data != "not classified" else ""
        tags = form.tags.data.split(",") if form.tags.data != "" else []
        note = DataObj(
            title=form.title.data,
            desc=form.desc.data,
            tags=tags,
            path=path,
            type="note")
        note_id = note.insert()
        if note_id:
            flash("Note Saved!")
            return redirect(f"/dataobj/{note_id}")
    return render_template(
        "/dataobjs/new.html",
        title="New Note",
        form=form)


@app.route("/dataobj/<dataobj_id>")
def show_dataobj(dataobj_id):
    dataobj = data.get_item(dataobj_id)

    if not dataobj:
        flash("Data could not be found!")
        return redirect("/")

    if request.args.get("raw") == "1":
        return frontmatter.dumps(dataobj)

    return render_template(
        "dataobjs/show.html",
        title=dataobj["title"],
        dataobj=dataobj,
        form=forms.DeleteDataForm())


@app.route("/dataobj/delete/<dataobj_id>", methods=["DELETE", "GET"])
def delete_data(dataobj_id):
    try:
        data.delete_item(dataobj_id)
    except BaseException:
        flash("Data could not be found!")
        return redirect("/")
    flash("Data deleted!")
    return redirect("/")


@app.route("/login", methods=["GET", "POST"])
def login():
    form = forms.UserForm()
    if form.validate_on_submit():
        db = get_db()
        user = db.search((Query().username == form.username.data) & (Query().type == "user"))

        if user and check_password_hash(user[0]["hashed_password"], form.password.data):
            user = User.from_db(user[0])
            login_user(user, remember=True)
            flash("Login successful!")

            next_url = request.args.get("next")
            return redirect(next_url or "/")

        flash("Invalid credentials")
        return redirect("/login")
    return render_template("users/form.html", form=form, title="Login")


@app.route("/logout", methods=["DELETE"])
@login_required
def logout():
    logout_user()
    flash("Logged out successfully")
    return redirect("/")


@app.route("/user/edit", methods=["GET", "POST"])
@login_required
def edit_user():
    form = forms.UserForm()
    if form.validate_on_submit():
        db = get_db()
        db.update(
            {
                "username": form.username.data,
                "hashed_password": generate_password_hash(form.password.data)
            },
            doc_ids=[current_user.id]
        )
        flash("Information saved!")
        return redirect("/")
    form.username.data = current_user.username
    return render_template("users/form.html", title="Edit Profile", form=form)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('requirements.txt', encoding='utf-8') as f:
    all_reqs = f.read().split('\n')
    install_requires = [x.strip() for x in all_reqs if not x.startswith('#')
                        and not x.startswith("-e git")]

setuptools.setup(
    name="archivy",
    version="0.10.0",
    author="Uzay-G",
    author_email="uzgirit@gmail.com",
    description=(
        "Minimalist knowledge base focused on digital preservation"
        " and building your second brain."
    ),
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Uzay-G/archivy",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License"
    ],
    entry_points={
        "console_scripts": [
            "archivy = archivy.cli:cli",
        ]
    },
    include_package_data=True,
    zip_safe=False,
    install_requires=install_requires,
    python_requires='>=3.6',
)



import os
import sys

import elasticsearch
import yaml
from elasticsearch import Elasticsearch
from flask import current_app, g
from tinydb import TinyDB, Query, operations


def load_config(path=""):
    """Loads `config.yml` file and deserializes it to a python dict."""
    path = path or current_app.config["INTERNAL_DIR"]
    with open(os.path.join(path, "config.yml")) as f:
        return yaml.load(f.read(), Loader=yaml.FullLoader)


def write_config(config: dict):
    """Writes a new config dict to a `config.yml` file that will override defaults"""
    with open(os.path.join(current_app.config["INTERNAL_DIR"], "config.yml"), "w") as f:
        yaml.dump(config, f)


def get_db(force_reconnect=False):
    """
    Returns the database object that you can use to
    store data persistently
    """
    if 'db' not in g or force_reconnect:
        g.db = TinyDB(
            os.path.join(
                current_app.config["INTERNAL_DIR"],
                "db.json"
            )
        )

    return g.db


def get_max_id():
    """Returns the current maximum id of dataobjs in the database."""
    db = get_db()
    max_id = db.search(Query().name == "max_id")
    if not max_id:
        db.insert({"name": "max_id", "val": 0})
        return 0
    return max_id[0]["val"]


def set_max_id(val):
    """Sets a new max_id"""
    db = get_db()
    db.update(operations.set("val", val), Query().name == "max_id")


def get_elastic_client():
    """Returns the elasticsearch client you can use to search and insert / delete data"""
    if not current_app.config["SEARCH_CONF"]["enabled"]:
        return None

    es = Elasticsearch(current_app.config["SEARCH_CONF"]["url"])
    try:
        health = es.cluster.health()
    except elasticsearch.exceptions.ConnectionError:
        current_app.logger.error(
            "Elasticsearch does not seem to be running on "
            f"{current_app.config['SEARCH_CONF']['url']}. Please start "
            "it, for example with: sudo service elasticsearch restart"
        )
        current_app.logger.error(
            "You can disable Elasticsearch by modifying the `enabled` variable "
            f"in {os.path.join(current_app.config['INTERNAL_DIR'], 'config.yml')}"
        )
        sys.exit(1)

    if health["status"] not in ("yellow", "green"):
        current_app.logger.warning(
            "Elasticsearch reports that it is not working "
            "properly. Search might not work. You can disable "
            "Elasticsearch by setting ELASTICSEARCH_ENABLED to 0."
        )
    return es
